#include <stdlib.h>
#include <memory.h>

#include "buffer.h"

static char hexString[] = "0123456789ABCDEF";

Buffer::Buffer( int c, int d ) :
  curLen( c ),
  pos( 0 ),
  delta( d )
{
  if( !delta )
    delta = 10;

  if( !curLen )
    curLen = delta;

  str = new char[ curLen ];
  reset();
}

void Buffer::putInto( char ch )
{
  if( pos==curLen )
  {
    char *old( str );

    str = new char [ curLen+delta ];

    memcpy( str, old, curLen );
    memset( str+curLen, 0, delta );

    curLen += delta;
    delete old;
  }

  str[ pos++ ] = ch;
}

void Buffer::put( register int ch )
{
  switch( ch )
  {

  case 0:
    putInto( 0 );
    break;

  case '\n':
    break;

  case '\r':
    putInto( '|' );
    break;

  default:
    if( ch<0x20 || ch>0x7f )
    {
      putInto( '\\' );
      putInto( 'x' );
      putInto( hexString[ (ch>>4)&0x0f ] );
      putInto( hexString[ ch&0x0f ] );

    }
    else
      putInto( char(ch) );
  }
}

void Buffer::putString( register char *str )
{
  for( ; *str; putInto( *(str++) ) );
}

void Buffer::putEMSI( register char *str )
{
  for( register char ch(0); ch=*str, ch; str++ )
    if( ch==']' || ch=='}' )
    {
      putInto( ch );
      putInto( ch );
    }
    else if( ch<0x20 || ch>0x7e || ch=='\\' )
    {
      putInto( '\\' );
      putInto( hexString[ (ch>>4)&0x0f ] );
      putInto( hexString[ ch&0x0f ] );
    }
    else
      putInto( ch );
}

void Buffer::stripLider()
{
  register char strip(1);
  register int j(0);

  for( register int i(0); i<pos; i++ )
    if( strip )
    {
      if( str[i]!='|' )
      {
	strip = 0;
	str[j++] = str[i];
      }
    }
    else
      str[j++] = str[i];

  if( j!=pos )
  {
    memset( str+j, 0, pos-j );
    pos = j;
  }
}

void Buffer::stripTrailer()
{
  register int i( pos-2 );

  for( ; i>=0 && str[i]=='|'; i-- );

  if(  pos-i-1>0 )
    memset( str+i+1, 0, pos-i-1 );

  pos = i+2;
}

void Buffer::reset()
{
  pos = 0;
  memset( str, 0, curLen );
}

void Buffer::leftShift( int how )
{
  if( how>pos )
    return;
  else
    if( how!=pos )
      memcpy( str, str+how, pos-how );

  pos -= how;
  memset( str+pos, 0, curLen-pos );
}

int Buffer::operator[]( int index )
{
  if( index>=pos )
    return -1;
  else
    return int(str[index])&0xff;
}
